SystemOrganization addCategory: #'XPForums-Networking-Core'!
SystemOrganization addCategory: #'XPForums-Networking-Server'!
SystemOrganization addCategory: #'XPForums-Networking-Entities'!
SystemOrganization addCategory: #'XPForums-Networking-EntityCommands'!
SystemOrganization addCategory: #'XPForums-Networking-TcpCommands'!

!Dictionary methodsFor: '*xpforums-networking-core' stamp: 'SR 9/9/2009 12:52'!
deserializeFrom: aStream
	aStream next. "$("
	[aStream peek = $)] whileFalse: [| key value |
		aStream next. "$<"
		key := aStream upToAll: '->'.
		value := aStream upTo: '>'.
		self add: key->value.
	].
	aStream next. "$)"! !

!Dictionary methodsFor: '*xpforums-networking-core' stamp: 'SR 9/9/2009 15:38'!
serializeOn: aStream
	aStream nextPut: $(.
	self keysSortedSafely do: [:key ||value|
		value := self at: key.
		value isString ifTrue: [value := value asHtml].
		aStream
			nextPut: $<;
			nextPutAll: key asString;
			nextPutAll: '->';				
			nextPutAll: value asString;
			nextPut: $>].
	aStream nextPut: $)! !

!WriteStream methodsFor: '*xpforums-networking' stamp: 'SR 9/8/2009 13:39'!
crlf
	"Append a return character to the receiver."

	self nextPut: Character cr.
	self nextPut: Character lf.! !

Object subclass: #XpfEntity
	instanceVariableNames: 'data uuid commandList'
	classVariableNames: 'UpstreamServer EntityList'
	poolDictionaries: ''
	category: 'XPForums-Networking-Core'!

!XpfEntity class methodsFor: 'accessing' stamp: 'SR 9/8/2009 18:57'!
EntityList

	EntityList ifNil: [EntityList := Set new].
	^ EntityList! !

!XpfEntity class methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 16:01'!
clearEntityList

	EntityList := nil.! !

!XpfEntity class methodsFor: 'accessing' stamp: 'SR 9/9/2009 15:57'!
getEntity: aUUID

	self EntityList do: [:e |
		(e uuid = aUUID) ifTrue: [^ e]].
	
	"Entity not found"
	self upstreamServerAvailable 
		ifTrue: [|entity|
			entity := self retrieveFromUpstreamServer: aUUID.
			entity ifNil: [^ nil].
			self EntityList add: entity.
			^ entity]
		ifFalse: [^ nil]! !

!XpfEntity class methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:43'!
primitiveAttributes

	^ {}! !

!XpfEntity class methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:43'!
referenceAttributes

	^ {}! !

!XpfEntity class methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 13:51'!
retrieveFromUpstreamServer: aUUID

	^ nil! !

!XpfEntity class methodsFor: 'accessing' stamp: 'SR 9/9/2009 13:51'!
upstreamServer

	^ UpstreamServer! !

!XpfEntity class methodsFor: 'accessing' stamp: 'SR 9/9/2009 13:51'!
upstreamServer: anObject

	UpstreamServer := anObject! !

!XpfEntity class methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 13:51'!
upstreamServerAvailable

	^ self upstreamServer isNil not! !

!XpfEntity methodsFor: 'commands' stamp: 'SR 9/9/2009 15:34'!
applyCommand: aCommand

	| result |
	result := aCommand apply: self.
	self commandList add: aCommand.
	^ result! !

!XpfEntity methodsFor: 'accessing' stamp: 'SR 9/8/2009 14:43'!
commandList

	^ commandList! !

!XpfEntity methodsFor: 'accessing' stamp: 'SR 9/8/2009 14:43'!
commandList: anObject

	commandList := anObject! !

!XpfEntity methodsFor: 'accessing' stamp: 'SR 9/8/2009 11:46'!
data

	^ data! !

!XpfEntity methodsFor: 'accessing' stamp: 'SR 9/8/2009 11:46'!
data: anObject

	data := anObject! !

!XpfEntity methodsFor: 'printing' stamp: 'SR 9/9/2009 13:11'!
deserializeFrom: stream

	| typeName |
	stream next. "${"
	typeName := stream upTo: '-'.
	type := (Smalltalk at: typeName asSymbol ifAbsent: [^ self]) new.	
	data := Dictionary new deserializeFrom: stream.
	stream next. "$}"! !

!XpfEntity methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:50'!
doesNotUnderstand: aMessage

	"handle getters"
	(self includesProperty: aMessage selector)
		ifTrue: [^ self getProperty: aMessage selector].

	" handle setters"
	((aMessage selector endsWith: ':') and:
		[self includesProperty: aMessage selector allButLast])
			ifTrue: [^ self setProperty: aMessage selector allButLast withValue: aMessage arguments first].

	"handle unknown"
	super doesNotUnderstand: aMessage.! !

!XpfEntity methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:49'!
getProperty: aSymbol

	(self class primitiveAttributes includes: aSymbol) ifTrue: [
		^ self at: aSymbol].
	
	(self class referenceAttributes includes: aSymbol) ifTrue: [
		^ XpfEntity getEntity: (UUID fromString: (self at: aSymbol))].
	
	^ self
	
	! !

!XpfEntity methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:44'!
includesProperty: aSymbol

	(self class primitiveAttributes includes: aSymbol) ifTrue: [^ true].
	(self class referenceAttributes includes: aSymbol) ifTrue: [^ true].
	
	^ false! !

!XpfEntity methodsFor: 'initialization' stamp: 'SR 9/9/2009 16:02'!
initialize

	super initialize.
	
	self
		data: Dictionary new;
		commandList: OrderedCollection new.! !

!XpfEntity methodsFor: 'testing' stamp: 'SR 9/9/2009 13:27'!
isListEntity

	^ false! !

!XpfEntity methodsFor: 'printing' stamp: 'SR 9/9/2009 15:45'!
printOn: stream

	stream nextPutAll: self class name.
	stream nextPut: $<.
	stream nextPutAll: self uuid asString.
	stream nextPut: $>.! !

!XpfEntity methodsFor: 'printing' stamp: 'SR 9/9/2009 15:26'!
serializeOn: stream

	stream nextPut: ${.
	stream nextPutAll: self class name asString.
	stream nextPut: $-.
	self data serializeOn: stream.
	stream nextPut: $}.
	! !

!XpfEntity methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 18:35'!
setProperty: aSymbol withValue: anObject

	| command |
	command := XpfSetAttributeCommand new.
	command attribute: aSymbol.
	command newValue: anObject.
			
	XpfEntity upstreamServerAvailable
		ifTrue: [XpfEntity upstreamServer sendCommand: command]
		ifFalse: [self applyCommand: command]
		! !

!XpfEntity methodsFor: 'accessing' stamp: 'SR 9/8/2009 14:40'!
uuid

	uuid ifNil: [
		uuid := UUID new].
	^ uuid! !

!XpfEntity methodsFor: 'accessing' stamp: 'SR 9/8/2009 14:47'!
version

	^ self commandList size! !

XpfEntity subclass: #XpfListEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-Entities'!

!XpfListEntity class methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 13:24'!
elementClass

	self subclassResponsibility! !

!XpfListEntity methodsFor: 'adding' stamp: 'SR 9/9/2009 13:41'!
add: aXpfObject

	(aXpfObject class = self class elementClass) ifFalse: [^ self].

	self data at: (self lastElement + 1) put: aXpfObject.
	self lastElement: self lastElement + 1.
	self size: self size + 1.	
	! !

!XpfListEntity methodsFor: 'adding' stamp: 'SR 9/9/2009 13:42'!
at: aNumber

	self data at: aNumber ifAbsent: [nil]! !

!XpfListEntity methodsFor: 'initialization' stamp: 'SR 9/9/2009 13:38'!
initialize

	super initialize.
	self data
		at: #size put: 0;
		at: #lastElement put: 0.! !

!XpfListEntity methodsFor: 'testing' stamp: 'SR 9/9/2009 13:27'!
isListEntity

	^ true! !

!XpfListEntity methodsFor: 'accessing' stamp: 'SR 9/9/2009 13:38'!
lastElement

	^ self data at: #lastElement ifAbsent: [0]! !

!XpfListEntity methodsFor: 'accessing' stamp: 'SR 9/9/2009 13:41'!
lastElement: aNumber

	self data at: #lastElement put: aNumber! !

!XpfListEntity methodsFor: 'accessing' stamp: 'SR 9/9/2009 13:37'!
size

	^ self data at: #size ifAbsent: [0]! !

!XpfListEntity methodsFor: 'accessing' stamp: 'SR 9/9/2009 13:41'!
size: aNumber

	self data at: #size put: aNumber! !

XpfListEntity subclass: #XpfProjectListEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-Entities'!

!XpfProjectListEntity class methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 13:25'!
elementClass

	^ XpfProjectEntity! !

!XpfProjectListEntity methodsFor: 'initialization' stamp: 'SR 9/8/2009 15:37'!
initialize

	super initialize.
	self type: XpfRootListType.! !

!XpfProjectListEntity methodsFor: 'accessing' stamp: 'SR 9/8/2009 15:27'!
uuid

	^ UUID fromString: '00000000-0000-0000-0000-000000000002'! !

XpfListEntity subclass: #XpfUserListEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-Entities'!

!XpfUserListEntity class methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 13:25'!
elementClass

	^ XpfUserEntity! !

!XpfUserListEntity methodsFor: 'initialization' stamp: 'SR 9/8/2009 15:37'!
initialize

	super initialize.
	self type: XpfRootListType.! !

!XpfUserListEntity methodsFor: 'accessing' stamp: 'SR 9/8/2009 15:27'!
uuid

	^ UUID fromString: '00000000-0000-0000-0000-000000000001'! !

XpfEntity subclass: #XpfProjectEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-Entities'!

XpfEntity subclass: #XpfUserEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-Entities'!

Object subclass: #XpfEntityCommand
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-Core'!

XpfEntityCommand subclass: #XpfCreateEntityCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-EntityCommands'!

!XpfCreateEntityCommand methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 18:37'!
apply: aXpfEntity

	aXpfEntity isListEntity
		ifFalse: [^ '403 Invalid Parameter']
		ifTrue: [ | newElement |
			newElement := aXpfEntity class elementClass new.
			XpfEntity EntityList add: newElement.
			aXpfEntity add: newElement.
			self newEntity: newElement.
			^ '250 ',newElement uuid asString]! !

!XpfCreateEntityCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 18:37'!
newEntity

	^ self at: #newEntity ifAbsent: [nil]! !

!XpfCreateEntityCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 18:37'!
newEntity: anObject

	^ self at: #newEntity put: anObject! !

!XpfEntityCommand class methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:33'!
getCommandFromString: aString

	| stream type parameter |
	stream := ReadStream on: aString.
	stream next. "{"
	type := Smalltalk at: (stream upTo: $-) asSymbol ifAbsent: [nil].
	type ifNil: [^ nil].
	parameter := Dictionary new deserializeFrom: stream.
	
	^ type new data: parameter.
	
	
	! !

!XpfEntityCommand methodsFor: 'command-pattern' stamp: 'SR 9/8/2009 14:44'!
apply: aXpfEntity

	self subclassResponsibility! !

!XpfEntityCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 12:57'!
data

	^ data! !

!XpfEntityCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 12:57'!
data: anObject

	data := anObject! !

!XpfEntityCommand methodsFor: 'initialization' stamp: 'SR 9/9/2009 12:57'!
initialize

	super initialize.
	self data: Dictionary new.! !

!XpfEntityCommand methodsFor: 'command-pattern' stamp: 'SR 9/8/2009 14:44'!
revert: aXpfEntity

	self subclassResponsibility! !

XpfEntityCommand subclass: #XpfSetAttributeCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-EntityCommands'!

!XpfSetAttributeCommand methodsFor: 'command-pattern' stamp: 'SR 9/9/2009 18:36'!
apply: aXpfEntity

	self oldValue: (aXpfEntity getProperty: self attribute).
	aXpfEntity data at: self attribute put: self newValue.
	! !

!XpfSetAttributeCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 18:37'!
attribute

	^ self data at: #attribute ifAbsent: [nil]! !

!XpfSetAttributeCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 18:31'!
attribute: aSymbol

	self data at: #attribute put: aSymbol! !

!XpfSetAttributeCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 18:37'!
newValue

	^ self data at: #newValue ifAbsent: [nil]! !

!XpfSetAttributeCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 18:32'!
newValue: anObject

	self data at: #newValue put: anObject! !

!XpfSetAttributeCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 18:37'!
oldValue

	^ self data at: #oldValue ifAbsent: [nil]! !

!XpfSetAttributeCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 18:32'!
oldValue: anObject

	self data at: #oldValue put: anObject! !

!XpfSetAttributeCommand methodsFor: 'command-pattern' stamp: 'SR 9/9/2009 18:33'!
revert: aXpfEntity

	aXpfEntity data at: self attribute put: self oldValue.
	! !

Object subclass: #XpfTcpCommand
	instanceVariableNames: 'finished response responseCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-Core'!

XpfTcpCommand subclass: #XpfTcpApplyCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-TcpCommands'!

!XpfTcpApplyCommand class methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 14:55'!
verb

	^ 'apply'! !

!XpfTcpApplyCommand methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:34'!
from: aEndpoint execute: line

	| uuid command entity return |
	[uuid := UUID fromString: (line truncateTo: 36).]
		on: Error
		do: [:ex| ^ '401 Parameter invalid'].

	command := XpfEntityCommand getCommandFromString: (line allButFirst: (37 min: line size)).
	command ifNil: [^ '402 Parameter invalid'].
	
	entity := XpfEntity getEntity: uuid.
	entity ifNil: [^ '400 Not Found'].
	return := entity applyCommand: command.
	
	self informCommandListenerOf: aEndpoint about: command on: entity.
	^ return.! !

!XpfTcpApplyCommand methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:14'!
informCommandListenerOf: endpoint about: command on: entity

	endpoint commandListener do: [:listener|[listener informAbout: command on: entity]
			on: Error
			do: []].! !

!XpfTcpCommand class methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 15:14'!
getCommandForLine: line

	self allSubclassesDo: [:subclass | 
		(line beginsWith: subclass verb) ifTrue: [
			^ subclass]].
	^ nil! !

!XpfTcpCommand class methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 14:55'!
verb

	^ self subclassResponsibility! !

!XpfTcpCommand methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:00'!
finishedSemaphore

	^ finished! !

!XpfTcpCommand methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 14:24'!
from: aEndpoint execute: line
	
	self subclassResponsibility! !

!XpfTcpCommand methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:00'!
initialize

	super initialize.
	finished := Semaphore new.! !

!XpfTcpCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 15:19'!
response

	^ response! !

!XpfTcpCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 15:19'!
response: anObject

	response := anObject! !

!XpfTcpCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 15:19'!
responseCode

	^ responseCode! !

!XpfTcpCommand methodsFor: 'accessing' stamp: 'SR 9/9/2009 15:19'!
responseCode: anObject

	responseCode := anObject! !

XpfTcpCommand subclass: #XpfTcpGetCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-TcpCommands'!

!XpfTcpGetCommand class methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 14:55'!
verb

	^ 'get'! !

!XpfTcpGetCommand methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:55'!
from: aEndpoint execute: line

	| uuid entity |
	[uuid := UUID fromString: line.]
		on: Error
		do: [:ex| ^ '401 Parameter invalid'].
	self halt.
	entity := XpfEntity getEntity: uuid.
	entity ifNil: [^ '400 Not Found'].
	
	^ String streamContents: [:stream| 
				stream print: 250.
				stream space.
				entity serializeOn: stream].
	! !

Object subclass: #XpfTcpEndpoint
	instanceVariableNames: 'socket stream workerProcess commandListener commandQueue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-Core'!

!XpfTcpEndpoint methodsFor: 'accessing' stamp: 'SR 9/9/2009 14:16'!
commandListener

	^ commandListener! !

!XpfTcpEndpoint methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:04'!
getEntity: aUUID

	| tcpCommand |
	tcpCommand := XpfTcpGetCommand new uuid: aUUID.
	tcpCommand send: self stream.
	commandQueue nextPut: tcpCommand.
	
	tcpCommand finishedSemaphore wait.
	(tcpCommand responseCode = 250)
		ifTrue: [^ tcpCommand response]
		ifFalse: [^ '']
	! !

!XpfTcpEndpoint methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 15:05'!
informCommandListener: aEntityCommand

	! !

!XpfTcpEndpoint methodsFor: 'initialization' stamp: 'SR 9/9/2009 14:54'!
initialize
	
	super initialize.
	commandListener := Set new.
	commandQueue := SharedQueue new.! !

!XpfTcpEndpoint methodsFor: 'private' stamp: 'SR 9/9/2009 15:22'!
processLine: aString

	|  |
	(aString beginsWith: '1')
		ifTrue: [ | line commandClass commandParameter |
			"Execute a command"
			line := aString allButFirst: (4 min: aString size).
			commandClass := XpfTcpCommand getCommandForLine: line.
			commandClass ifNil: [^ ''].
			commandParameter := line allButFirst: ((commandClass verb size + 1) min: line size).
			^ commandClass new from: self execute: commandParameter]
		ifFalse: [ | command |
			"Response to a former command"
			command := commandQueue nextOrNil.
			command ifNil: [^ nil].
			command responseCode: (aString truncateTo: 3) asInteger.
			command response: (aString allButFirst: (3 min: aString size)).
			command finishedSemaphore signal.
			^ nil].
! !

!XpfTcpEndpoint methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 14:58'!
sendCommand: aEntityCommand

	| tcpCommand |
	tcpCommand := XpfTcpApplyCommand new entityCommand: aEntityCommand.
	tcpCommand send: self stream.
	commandQueue nextPut: tcpCommand.! !

!XpfTcpEndpoint methodsFor: 'accessing' stamp: 'SR 9/9/2009 14:10'!
socket

	^ socket! !

!XpfTcpEndpoint methodsFor: 'accessing' stamp: 'SR 9/9/2009 14:11'!
socket: anObject

	socket := anObject.
	stream := SocketStream on: anObject.
	workerProcess := self workerBlock fork.! !

!XpfTcpEndpoint methodsFor: 'accessing' stamp: 'SR 9/9/2009 14:11'!
stream

	^ stream! !

!XpfTcpEndpoint methodsFor: 'controlling' stamp: 'SR 9/9/2009 14:07'!
terminate
	"Compliance to Process-protocoll in order to function as client-worker too"! !

!XpfTcpEndpoint methodsFor: 'private' stamp: 'SR 9/9/2009 14:13'!
workerBlock

	^ [
		[self socket notNil and: [self stream isConnected]] whileTrue: [|data response|
	      	data := self stream upToAll: String crlf.
			data = '.'
				ifTrue: [self socket closeAndDestroy. response := nil]
				ifFalse: [response := self processLine: data].
			response isNil ifFalse: [
           		stream
             			nextPutAll: response;
             			crlf;
             			flush
			].
		].
	]! !

!XpfTcpEndpoint methodsFor: 'accessing' stamp: 'SR 9/9/2009 14:11'!
workerProcess

	^ workerProcess! !

Object subclass: #XpfTcpTextServer
	instanceVariableNames: 'port listenSocket clientWorkerList listenProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-Server'!

XpfTcpTextServer subclass: #XpfTcpCentralServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-Server'!

!XpfTcpCentralServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 14:48'!
hasWelcomeBanner

	^ false! !

!XpfTcpCentralServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 19:01'!
initialize

	super initialize.
	self
		port: 54300;
		initializeEntityList.! !

!XpfTcpCentralServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 19:00'!
initializeEntityList

	XpfEntity EntityList isEmpty ifTrue: [
		XpfEntity EntityList
			add: XpfUserListEntity new;
			add: XpfProjectListEntity new].! !

!XpfTcpCentralServer methodsFor: 'as yet unclassified' stamp: 'SR 9/9/2009 14:08'!
processSocket: aSocket
 
	^ XpfTcpEndpoint new socket: aSocket.! !

XpfTcpTextServer subclass: #XpfTcpEchoServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPForums-Networking-Server'!

!XpfTcpEchoServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 12:18'!
processLine: aString

	Transcript show: aString; cr.
	^ aString! !

!XpfTcpTextServer methodsFor: 'accessing' stamp: 'SR 9/8/2009 12:03'!
clientWorkerList

	^ clientWorkerList! !

!XpfTcpTextServer methodsFor: 'accessing' stamp: 'SR 9/8/2009 12:03'!
clientWorkerList: anObject

	clientWorkerList := anObject! !

!XpfTcpTextServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 13:25'!
hasWelcomeBanner

	^ false! !

!XpfTcpTextServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 12:03'!
initialize

	super initialize.
	Socket initializeNetwork.
	
	self
		port: 54320;
		clientWorkerList: OrderedCollection new.! !

!XpfTcpTextServer methodsFor: 'accessing' stamp: 'SR 9/8/2009 12:16'!
listenProcess

	^ listenProcess! !

!XpfTcpTextServer methodsFor: 'accessing' stamp: 'SR 9/8/2009 12:16'!
listenProcess: anObject

	listenProcess := anObject! !

!XpfTcpTextServer methodsFor: 'accessing' stamp: 'SR 9/8/2009 11:55'!
port

	^ port! !

!XpfTcpTextServer methodsFor: 'accessing' stamp: 'SR 9/8/2009 11:55'!
port: anObject

	port := anObject! !

!XpfTcpTextServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 11:53'!
processLine: aString

	self subclassResponsibility.! !

!XpfTcpTextServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 13:32'!
processSocket: aSocket
 	
	^ [| stream |      
		stream := SocketStream on: aSocket.
		
		self hasWelcomeBanner ifTrue: [
			stream nextPutAll: self welcomeBanner; crlf; flush].
	
	    	[aSocket notNil and: [stream isConnected]] whileTrue: [|data response|
	      	data := stream upToAll: String crlf.
			data = '.'
				ifTrue: [aSocket closeAndDestroy. response := nil]
				ifFalse: [response := self processLine: data].
			response isNil ifFalse: [
           		stream
             			nextPutAll: response;
             			crlf;
             			flush
			].
       	].
       	aSocket ifNotNil: [aSocket closeAndDestroy].
	] fork.! !

!XpfTcpTextServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 12:15'!
running

	^ self listenProcess isNil not! !

!XpfTcpTextServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 12:17'!
startServer

	| |
	self running ifTrue: [^ self].
	
	listenSocket := Socket newTCP.
	listenSocket listenOn: (self port) backlogSize: 4.
   	self listenProcess: (
		[
			[true] whileTrue: [ | socket |
				socket := listenSocket waitForAcceptFor: 60.
				self clientWorkerList add: (self processSocket: socket).
		
			]
		] fork).! !

!XpfTcpTextServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 12:17'!
stopServer

	self running ifFalse: [^ self].
	
	self listenProcess terminate.
	listenSocket closeAndDestroy.
	
	self clientWorkerList do: [:element |
		element isTerminated ifFalse: [element terminate]].! !

!XpfTcpTextServer methodsFor: 'as yet unclassified' stamp: 'SR 9/8/2009 13:25'!
welcomeBanner

	^ self subclassResponsibility! !
